unit KOLTracker;

interface

uses Windows, Messages, KOL;

type
  PTracker = ^TTracker;
  TTracker = object( TControl )
  protected
    function Get(const Index: Integer): Integer;
    procedure Put(const Index, Value: Integer);
    procedure PaintTracker( Sender: PControl; DC: HDC );
  public
    property MinPos: Integer index 0 read Get write Put;
    property MaxPos: Integer index 4 read Get write Put;
    property CurPos: Integer index 8 read Get write Put;
    property DfltPos: Integer index 12 read Get write Put;
    procedure ReleaseCapture;
  end;

function NewTracker( AParent: PControl ): PTracker;

{$IFDEF KOL_MCK}
type
  TKOLTracker = PTracker;
{$ENDIF}

implementation

type
  PTRackerData = ^TTrackerData;
  TTrackerData = packed record
    MinPos, MaxPos: Integer;
    CurPos: Integer;
    DfltPos: Integer;
    LastDblClkTime: DWORD;
    Captured: Boolean;
  end;

function WndProcTracker( Sender: PControl; var M: TMsg; var Rslt: Integer ): Boolean;
var i, n: Integer;
    D: PTRackerData;
begin
  Result := FALSE;
  CASE M.message OF
  WM_MOUSEMOVE, WM_LBUTTONDOWN:
    if M.wParam = MK_LBUTTON then
    begin
      D := Sender.CustomData;
      if M.message = WM_LBUTTONDOWN then
      begin
        SetCapture( Sender.Handle );
        D.Captured := TRUE;
      end;
      if GetTickCount - D.LastDblClkTime > 500 then
      begin
        n := Sender.Height div 2;
        i := SmallInt( LoWord( M.lParam ) );
        i := max( i, n );
        i := min( i, Sender.Width-n-1 );
        i := D.MinPos + (i-n) * (D.MaxPos - D.MinPos) div max( 1, (Sender.Width - n * 2 - 1) );
        if D.CurPos <> i then
        begin
          D.CurPos := i;
          Sender.Invalidate;
          if Assigned( Sender.OnChange ) then
            Sender.OnChange( Sender );
        end;
      end;
    end;
  WM_LBUTTONUP:
    begin
      D := Sender.CustomData;
      if D.Captured then
      begin
        D.Captured := FALSE;
        ReleaseCapture;
      end;
    end;
  WM_LBUTTONDBLCLK:
    begin
      D := Sender.CustomData;
      if (D.DfltPos >= D.MinPos) and (D.DfltPos <= D.MaxPos) then
      begin
        D.LastDblClkTime := GetTickCount;
        if D.CurPos <> D.DfltPos then
        begin
          D.CurPos := D.DfltPos;
          Sender.Invalidate;
          if Assigned( Sender.OnChange ) then
            Sender.OnChange( Sender );
        end;
      end;
    end;
  END;
end;

function NewTracker( AParent: PControl ): PTracker;
var D: PTRackerData;
begin
  Result := PTracker( NewPaintbox( AParent ) );
  D := AllocMem( Sizeof( D^ ) );
  Result.CustomData := D;
  D.MaxPos := 100;
  D.DfltPos := -1;
  Result.Width := 104;
  Result.Height := 8;
  Result.OnPaint := PTracker( Result ).PaintTracker;
  Result.AttachProc( WndProcTracker );
end;

{ TKOLTracker }

function TTracker.Get(const Index: Integer): Integer;
var D: PTRackerData;
begin
  D := CustomData;
  Result := PInteger( Integer( D ) + Index )^;
end;

procedure TTracker.PaintTracker(Sender: PControl; DC: HDC);
var TmpBmp: PBitmap;
    C: PCanvas;
    i, h, w, n: Integer;
    D: PTRackerData;
begin
  D := Sender.CustomData;
  TmpBmp := nil;
  h := Sender.Height;
  w := Sender.Width;
  n := h div 2;
  if not Sender.Transparent then
  begin
    TmpBmp := NewDibBitmap( w, h, pf32bit );
    C := TmpBmp.Canvas;
    C.Brush.Color := Sender.Color;
    C.FillRect( TmpBmp.BoundsRect );
  end
    else
  begin
    C := Sender.Canvas;
  end;
  TRY
    i := (w - 1 - n*2) * (D.CurPos - D.MinPos) div (D.MaxPos - D.MinPos) + n;
    C.Brush.Color := clWhite;
    C.Pen.Color := clBlack;
    if i > 2 then
    begin
      C.MoveTo( 0, h-3 );
      C.LineTo( i-1, h-3 );
    end;
    if i < w-2 then
    begin
      C.MoveTo( w, h-3 );
      C.LineTo( i+1, h-3 );
    end;
    C.Polygon( [ MakePoint( i, 0 ),
               MakePoint( i-n, h-1 ), MakePoint( i+n, h-1 ) ] );
    if TmpBmp <> nil then
      TmpBmp.Draw( DC, 0, 0 );
  FINALLY
    TmpBmp.Free;
  END;
end;

procedure TTracker.Put(const Index, Value: Integer);
var D: PTRackerData;
begin
  D := CustomData;
  PInteger( Integer( D ) + Index )^ := Value;
  Invalidate;
end;

procedure TTracker.ReleaseCapture;
var D: PTRackerData;
begin
  D := CustomData;
  if D.Captured then
  begin
    D.Captured := FALSE;
    ReleaseCapture;
  end;
end;

end.
