unit MCKTracker;

interface

uses Windows, Messages, mirror, classes, Controls, Forms, kol, mckCtrls,
  KOLTracker, SysUtils;

type
  TKOLTracker = class( TKOLControl )
  private
    FCurPos: Integer;
    FMaxPos: Integer;
    FMinPos: Integer;
    FDfltPos: Integer;
    procedure SetCurPos(const Value: Integer);
    procedure SetMaxPos(const Value: Integer);
    procedure SetMinPos(const Value: Integer);
    procedure SetDfltPos(const Value: Integer);
  protected
    procedure SetupFirst( SL: TStringList; const AName, AParent, Prefix: String ); override;
    function AdditionalUnits: String; override;
    procedure SetupConstruct( SL: TStringList; const AName, AParent, Prefix: String ); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds( aLeft, aTop, aWidth, aHeight: Integer ); override;
  published
    property MinPos: Integer read FMinPos write SetMinPos;
    property MaxPos: Integer read FMaxPos write SetMaxPos;
    property CurPos: Integer read FCurPos write SetCurPos;
    property DfltPos: Integer read FDfltPos write SetDfltPos;
    property OnChange;
    property Transparent;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('KOL', [TKOLTracker]);
end;


{ TKOLTracker }

function TKOLTracker.AdditionalUnits: String;
begin
  Result := ', KOLTracker';
end;

constructor TKOLTracker.Create(AOwner: TComponent);
begin
   inherited;
   Constraints.MaxHeight := 40;
   FMaxPos := 100;
   FDfltPos := -1;
   FHasBorder := FALSE;
   FDefHasBorder := FALSE;
   DefaultWidth := 108;
   DefaultHeight := 8;
   SetBounds( Left, Top, 108, 8 );
end;

procedure TKOLTracker.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  if aHeight > 40 then
    aHeight := 8;
  // Who was the GOAT in the Borland desiding that a control never
  // will be created with the initial Height < 10 ????
  inherited;
  {Rpt( 'Assigning bounds: ' + IntToStr( aLeft ) + ',' + IntToStr( aTop ) + ',' +
    IntToStr( aWidth ) + ',' + IntToStr( aHeight ), RED );}
  {if aHeight > 40 then
  begin
    Rpt( 'Assigning bounds (h>40): ' + IntToStr( aLeft ) + ',' + IntToStr( aTop ) + ',' +
      IntToStr( aWidth ) + ',' + IntToStr( aHeight ), CYAN );
    //Rpt_Stack;
  end;}
end;

procedure TKOLTracker.SetCurPos(const Value: Integer);
begin
  if FCurPos = Value then Exit;
  FCurPos := Value;
  if FCurPos < FMinPos then FCurPos := FMinPos;
  if FCurPos > FMaxPos then FCurPos := FMaxPos;
  Change;
end;

procedure TKOLTracker.SetDfltPos(const Value: Integer);
begin
  if FDfltPos = Value then Exit;
  FDfltPos := Value;
  Change;
end;

procedure TKOLTracker.SetMaxPos(const Value: Integer);
begin
  if FMaxPos = Value then Exit;
  FMaxPos := Value;
  if FMaxPos <= FMinPos then
    FMaxPos := FMinPos + 1;
  if FCurPos > FMaxPos then FCurPos := FMaxPos;
  Change;
end;

procedure TKOLTracker.SetMinPos(const Value: Integer);
begin
  if FMinPos = Value then Exit;
  FMinPos := Value;
  if FMinPos >= FMaxPos then
    FMinPos := FMaxPos-1;
  if FCurPos < FMinPos then FCurPos := FMinPos;
  Change;
end;

procedure TKOLTracker.SetupConstruct(SL: TStringList; const AName, AParent,
  Prefix: String);
var S: String;
begin
  S := GenerateTransparentInits;
  SL.Add( Prefix + AName + ' := NewTracker( '
          + SetupParams( AName, AParent ) + ' );' );
  if S <> '' then
    SL.Add( Prefix + AName + S + ';' );
end;

procedure TKOLTracker.SetupFirst(SL: TStringList; const AName, AParent,
  Prefix: String);
begin
  inherited;
  if FMinPos <> 0 then
    SL.Add( Prefix + AName + '.MinPos := ' + IntToStr( FMinPos ) + ';' );
  if FMaxPos <> 100 then
    SL.Add( Prefix + AName + '.MaxPos := ' + IntToStr( FMaxPos ) + ';' );
  if FCurPos <> 0 then
    SL.Add( Prefix + AName + '.CurPos := ' + IntToStr( FCurPos ) + ';' );
  if (FDfltPos >= FMinPos) and (FDfltPos <= FMaxPos) then
    SL.Add( Prefix + AName + '.DfltPos := ' + IntToStr( FDfltPos ) + ';' );
end;

end.
